@@ -29,7 +29,8 @@
 #include "support.h"
 #include "window.h"
 
-#define INDEX (archive->type == XARCHIVETYPE_RAR ? (archive->tag == 5 ? 1 : 0) : 0)
+#define INDEX (archive->type == XARCHIVETYPE_RAR ? (TAGTYPE(archive->tag) == 4 ? 0 : 1) : 0)
+#define READ_ONLY (TAGTYPE(archive->tag) == 'x')   // exe
 
 static gboolean data_line, encrypted, last_line;
 
@@ -246,14 +247,16 @@ void xa_7zip_ask (XArchive *archive)
 
 	archive->can_test = TRUE;
 	archive->can_extract = TRUE;
-	archive->can_add = archiver[archive->type].is_compressor;
-	archive->can_delete = (archiver[archive->type].is_compressor && !SINGLE_FILE_COMPRESSOR(archive));
+	archive->can_add = (archiver[archive->type].is_compressor && !READ_ONLY);
+	archive->can_delete = (archiver[archive->type].is_compressor && !SINGLE_FILE_COMPRESSOR(archive) && !READ_ONLY);
 	archive->can_sfx = (archive->type == XARCHIVETYPE_7ZIP);
 	archive->can_password = (archive->type == XARCHIVETYPE_7ZIP);
 	archive->can_full_path[0] = TRUE;
 	archive->can_overwrite = TRUE;
 	archive->can_update[1] = archiver[archive->type].is_compressor;
 	archive->can_freshen[1] = (archiver[archive->type].is_compressor && !SINGLE_FILE_COMPRESSOR(archive));
+	archive->can_recurse[0] = TRUE;
+	archive->can_recurse[1] = (archiver[archive->type].is_compressor ? FORCED : FALSE);
 	archive->can_move = archiver[archive->type].is_compressor;
 	archive->can_solid = (archive->type == XARCHIVETYPE_7ZIP);
 	archive->can_compress = archiver[archive->type].is_compressor;
@@ -263,8 +266,16 @@ void xa_7zip_ask (XArchive *archive)
 
 static gchar *xa_7zip_password_str (XArchive *archive)
 {
+	gchar *escaped, *password_str;
+
 	if (archive->password)
-		return g_strconcat(" -p", archive->password, NULL);
+	{
+		escaped = xa_escape_bad_chars(archive->password, ESCAPES);
+		password_str = g_strconcat(" -p", escaped, NULL);
+		g_free(escaped);
+
+		return password_str;
+	}
 	else
 		return g_strdup("");
 }
@@ -394,7 +405,7 @@ void xa_7zip_list (XArchive *archive)
 			return;
 
 	/* a single file compressor archive is no longer new and empty now */
-	archive->can_add = (archiver[archive->type].is_compressor && !SINGLE_FILE_COMPRESSOR(archive));
+	archive->can_add = (archiver[archive->type].is_compressor && !SINGLE_FILE_COMPRESSOR(archive) && !READ_ONLY);
 
 	data_line = FALSE;
 	last_line = FALSE;
@@ -453,7 +464,7 @@ gboolean xa_7zip_extract (XArchive *archive, GSList *file_list)
 	gchar *password_str, *command;
 	gboolean result;
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
 	password_str = xa_7zip_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[INDEX],
 	                      archive->do_full_path ? " x" : " e",
@@ -477,7 +488,7 @@ void xa_7zip_add (XArchive *archive, GSList *file_list)
 
 	compression = g_strdup_printf("%hu", archive->compression);
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
 	password_str = xa_7zip_password_str(archive);
 	solid = g_strconcat(" -ms=", archive->do_solid ? "on" : "off", NULL);
 	command = g_strconcat(archiver[archive->type].program[0],
@@ -501,7 +512,7 @@ void xa_7zip_delete (XArchive *archive, GSList *file_list)
 	GString *files;
 	gchar *password_str, *command;
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
 	password_str = xa_7zip_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[0], " d", password_str, " -bd -spd -y ", archive->path[1], " --", files->str, NULL);
 	g_free(password_str);
