@@ -302,7 +302,7 @@ void xa_set_add_dialog_options(Add_dialog_data *add_dialog,XArchive *archive)
 	gtk_widget_set_sensitive(add_dialog->store_path, full_path);
 	gtk_widget_set_sensitive(add_dialog->update, archive->can_update[1]);
 	gtk_widget_set_sensitive(add_dialog->freshen, archive->can_freshen[1]);
-	gtk_widget_set_sensitive(add_dialog->recurse, !archive->can_recurse);
+	gtk_widget_set_sensitive(add_dialog->recurse, archive->can_recurse[1] != FORCED);
 	gtk_widget_set_sensitive(add_dialog->remove_files, archive->can_move);
 	gtk_widget_set_sensitive(add_dialog->solid_archive, archive->can_solid);
 
@@ -347,7 +347,7 @@ void xa_set_add_dialog_options(Add_dialog_data *add_dialog,XArchive *archive)
 void xa_parse_add_dialog_options (XArchive *archive,Add_dialog_data *add_dialog)
 {
 	gchar *temp_password = NULL;
-	gboolean done = FALSE;
+	gboolean done = FALSE, recurse;
 	GSList *list = NULL;
 
 	while ( ! done )
@@ -420,7 +420,16 @@ void xa_parse_add_dialog_options (XArchive *archive,Add_dialog_data *add_dialog)
 			if (!archive->do_full_path)
 				archive->child_dir = g_path_get_dirname(list->data);
 
-			xa_execute_add_commands(archive, list, gtk_widget_is_sensitive(add_dialog->recurse) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->recurse)), gtk_widget_is_sensitive(add_dialog->recurse));
+			recurse = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(add_dialog->recurse));
+			archive->do_recurse = FALSE;
+
+			if (archive->can_recurse[1])
+			{
+				archive->do_recurse = (recurse || (archive->can_recurse[1] == FORCED));
+				recurse = FALSE;
+			}
+
+			xa_execute_add_commands(archive, list, recurse, !archive->do_recurse);
 		}
 	}
 	gtk_widget_destroy(add_dialog->compression_scale);
@@ -514,6 +523,9 @@ void xa_execute_add_commands (XArchive *archive, GSList *list, gboolean recurse,
 
 	while (list)
 	{
+		if (archive->can_descend)
+			all = FALSE;
+
 		xa_recurse_local_directory((gchar*) list->data, &dirlist, archive->do_full_path, recurse, all);
 		list = list->next;
 	}
