@@ -335,6 +335,10 @@ gboolean xa_get_compressed_tar_type (XArchiveType *type)
 			*type = XARCHIVETYPE_TAR_LZOP;
 			break;
 
+		case XARCHIVETYPE_RZIP:
+			*type = XARCHIVETYPE_TAR_RZIP;
+			break;
+
 		case XARCHIVETYPE_XZ:
 			*type = XARCHIVETYPE_TAR_XZ;
 			break;
@@ -457,31 +461,6 @@ void xa_spawn_async_process (XArchive *archive, const gchar *command)
 		g_child_watch_add_full(G_PRIORITY_LOW, archive->child_pid, (GChildWatchFunc) xa_process_exit, archive, NULL);
 }
 
-/*	TODO: workaround for bug #3235
- *
- * gchar *xa_split_command_line(XArchive *archive,GSList *list)
-{
-	gchar *command = NULL;
-	GSList *chunks = NULL;
-	GSList *scan = NULL;
-	int length;
-
-	for (scan = list; scan != NULL;)
-	{
-		length = 0;
-		while ((scan != NULL) && (length < 5000)) //MAX_CMD_LEN
-		{
-			length += strlen (scan->data);
-			chunks = g_slist_prepend(chunks,scan->data);
-			scan = scan->next;
-		}
-		chunks = g_slist_prepend(chunks,"****** ");
-	}
-	chunks = g_slist_reverse(chunks);
-	return command;
-}
-*/
-
 void xa_clean_archive_structure (XArchive *archive)
 {
 	xa_free_entry(archive, archive->root_entry);
@@ -516,34 +495,37 @@ void xa_clean_archive_structure (XArchive *archive)
 gboolean xa_create_working_directory (XArchive *archive)
 {
 	gchar *tmp_dir;
-	gchar *value;
+	gchar *value, *value_local = NULL;
+	const gchar *tmp;
 
-	if (archive->working_dir != NULL)
+	if (archive->working_dir)
 		return TRUE;
 
 	value = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(prefs_window->combo_prefered_temp_dir));
 
 	if (value && *value)
 	{
-		gchar *value_local = g_filename_from_utf8(value, -1, NULL, NULL, NULL);
-		tmp_dir = g_strconcat(value_local, "/xa-XXXXXX", NULL);
-		g_free(value_local);
+		value_local = g_filename_from_utf8(value, -1, NULL, NULL, NULL);
+		tmp = value_local;
 	}
 	else
-		tmp_dir = g_strdup("");
+		tmp = g_get_tmp_dir();
 
+	tmp_dir = g_strconcat(tmp, "/xa-XXXXXX", NULL);
+
+	g_free(value_local);
 	g_free(value);
 
 	if (!g_mkdtemp(tmp_dir))
 	{
 		g_free(tmp_dir);
-		tmp_dir = NULL;
-
 		xa_show_message_dialog (GTK_WINDOW (xa_main_window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't create temporary directory:"),g_strerror(errno));
+
 		return FALSE;
 	}
 
 	archive->working_dir = tmp_dir;
+
 	return TRUE;
 }
 
@@ -857,7 +839,6 @@ void xa_dir_sidebar_row_selected (GtkTreeSelection *selection, gpointer user_dat
 	XEntry *entry;
 	GtkTreeIter iter;
 	GtkTreeIter parent;
-	GtkTreePath *path;
 	GtkTreeModel *model;
 	GString *string = g_string_new("");
 	gchar *dir;
@@ -867,10 +848,18 @@ void xa_dir_sidebar_row_selected (GtkTreeSelection *selection, gpointer user_dat
 
 	if ((idx != -1) && gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
-		path = gtk_tree_model_get_path(model,&iter);
-		if ( ! gtk_tree_view_row_expanded(GTK_TREE_VIEW(archive_dir_treeview),path))
-			gtk_tree_view_expand_to_path(GTK_TREE_VIEW(archive_dir_treeview),path);
-		gtk_tree_path_free(path);
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->auto_expand)))
+		{
+			GtkTreePath *path;
+
+			path = gtk_tree_model_get_path(model, &iter);
+
+			if (!gtk_tree_view_row_expanded(GTK_TREE_VIEW(archive_dir_treeview), path))
+				gtk_tree_view_expand_to_path(GTK_TREE_VIEW(archive_dir_treeview), path);
+
+			gtk_tree_path_free(path);
+		}
+
 		/* Let get the last selected dir */
 		gtk_tree_model_get(model,&iter,1,&dir,-1);
 		g_string_prepend_c(string,'/');
