@@ -33,6 +33,7 @@ typedef enum
 	XARCHIVETYPE_BZIP2,
 	XARCHIVETYPE_BZIP3,
 	XARCHIVETYPE_CAB,
+	XARCHIVETYPE_CHM,
 	XARCHIVETYPE_COMPRESS,
 	XARCHIVETYPE_CPIO,
 	XARCHIVETYPE_GZIP,
@@ -45,6 +46,7 @@ typedef enum
 	XARCHIVETYPE_LZOP,
 	XARCHIVETYPE_RAR,
 	XARCHIVETYPE_RPM,
+	XARCHIVETYPE_RZIP,
 	XARCHIVETYPE_SQUASHFS,
 	XARCHIVETYPE_TAR,
 	XARCHIVETYPE_TAR_BZIP,
@@ -57,6 +59,7 @@ typedef enum
 	XARCHIVETYPE_TAR_LZIP,
 	XARCHIVETYPE_TAR_LZMA,
 	XARCHIVETYPE_TAR_LZOP,
+	XARCHIVETYPE_TAR_RZIP,
 	XARCHIVETYPE_TAR_XZ,
 	XARCHIVETYPE_TAR_ZSTD,
 	XARCHIVETYPE_XZ,
@@ -121,6 +124,9 @@ typedef struct
 	gushort tag;
 } ArchiveType;
 
+#define TAG(t1,t2) ((t2 << 8) + t1)
+#define TAGTYPE(t) (t & 0xff)
+
 typedef struct _XEntry XEntry;
 
 struct _XEntry
@@ -193,9 +199,10 @@ struct _XArchive
 	gboolean can_overwrite;      // can overwrite files on extraction
 	gboolean can_update[2];      // can limit activity to only changed or new files
 	gboolean can_freshen[2];     // can limit activity to only changed files
+	gint can_recurse[2];         // can automatically recurse through subdirectories
 	gboolean can_move;           // can delete files after adding
 	gboolean can_solid;          // can create a solid archive
-	gboolean can_recurse;        // can automatically recurse through subdirectories
+	gboolean can_descend;        // can automatically descend to directories
 	gboolean can_compress;       // can compress at different levels
 	/* instructions */
 	gboolean do_full_path;   // extract_func, (xa_execute_add_commands)
@@ -203,6 +210,7 @@ struct _XArchive
 	gboolean do_overwrite;   // extract_func
 	gboolean do_update;      // extract_func, add_func
 	gboolean do_freshen;     // extract_func, add_func
+	gboolean do_recurse;     // extract_func, add_func
 	gboolean do_move;        // add_func
 	gboolean do_solid;       // add_func
 	/* child process */
@@ -214,7 +222,7 @@ struct _XArchive
 	gint child_fderr;
 	gchar *command;     // (free before use)
 	GSList *output;
-	/* (un)compressor interface */
+	/* (de)compressor interface */
 	XArchiver *archiver;
 	parse_output_func parse_output;
 	guint8 exitstatus_ok;
@@ -224,6 +232,16 @@ extern XArchive *archive[];
 
 #define can_rename(archive) ((archive)->can_extract && (archive)->can_delete && (archive)->can_add)
 
+/* how to specify the directory for can_recurse[0] */
+enum
+{
+	DIR_WITHOUT_SLASH,
+	DIR_WITH_SLASH,
+	DIR_WITH_ASTERISK
+};
+
+#define FORCED (TRUE + TRUE)   // third value for can_recurse[1], meaning that recursion is not optional
+
 typedef enum
 {
 	XA_CLIPBOARD_EMPTY,
