@@ -31,12 +31,15 @@ static gboolean data_line, fname_line;
 
 void xa_arj_ask (XArchive *archive)
 {
+	gboolean read_only;
 	compressor_t arj_compressor = {TRUE, 4, 1, 1, 1};
 
+	read_only = (archive->tag == 'x');   // exe
+
 	archive->can_test = TRUE;
 	archive->can_extract = TRUE;
-	archive->can_add = archiver[archive->type].is_compressor;
-	archive->can_delete = archiver[archive->type].is_compressor;
+	archive->can_add = (archiver[archive->type].is_compressor && !read_only);
+	archive->can_delete = (archiver[archive->type].is_compressor && !read_only);
 	archive->can_sfx = archiver[archive->type].is_compressor;
 	archive->can_password = archiver[archive->type].is_compressor;
 	archive->can_full_path[0] = archiver[archive->type].is_compressor;
@@ -46,7 +49,10 @@ void xa_arj_ask (XArchive *archive)
 	archive->can_update[1] = archiver[archive->type].is_compressor;
 	archive->can_freshen[0] = archiver[archive->type].is_compressor;
 	archive->can_freshen[1] = archiver[archive->type].is_compressor;
+	archive->can_recurse[0] = TRUE;
+	archive->can_recurse[1] = archiver[archive->type].is_compressor;
 	archive->can_move = archiver[archive->type].is_compressor;
+	archive->can_descend = archiver[archive->type].is_compressor;
 	archive->can_compress = archiver[archive->type].is_compressor;
 	archive->compressor = arj_compressor;
 	archive->compression = archive->compressor.preset;
@@ -54,8 +60,16 @@ void xa_arj_ask (XArchive *archive)
 
 static gchar *xa_arj_password_str (XArchive *archive)
 {
+	gchar *escaped, *password_str;
+
 	if (archive->password && archiver[archive->type].is_compressor)
-		return g_strconcat(" -g", archive->password, NULL);
+	{
+		escaped = xa_escape_bad_chars(archive->password, ESCAPES);
+		password_str = g_strconcat(" -g", escaped, NULL);
+		g_free(escaped);
+
+		return password_str;
+	}
 	else
 		return g_strdup("");
 }
@@ -263,7 +277,7 @@ gboolean xa_arj_extract (XArchive *archive, GSList *file_list)
 	gchar *command;
 	gboolean result = FALSE;
 
-	files = xa_quote_filenames(file_list, "*?[]", FALSE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_ASTERISK);
 
 	if (archiver[archive->type].is_compressor)
 	{
@@ -271,6 +285,7 @@ gboolean xa_arj_extract (XArchive *archive, GSList *file_list)
 		command = g_strconcat(archiver[archive->type].program[0],
 		                      archive->do_full_path ? " x" : " e",
 		                      archive->do_overwrite ? "" : (archive->do_update ? " -u" : (archive->do_freshen ? " -f" : " -n")),
+		                      archive->do_recurse ? " -p1" : "",
 		                      password_str, " -i -y ",
 		                      archive->path[1], " ",
 		                      archive->extraction_dir, " --", files->str, NULL);
@@ -351,10 +366,11 @@ void xa_arj_add (XArchive *archive, GSList *file_list)
 
 	compression = g_strdup_printf("%hu", archive->compression);
 
-	files = xa_quote_filenames(file_list, "*?[]", FALSE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
 	password_str = xa_arj_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[0],
 	                      archive->do_update ? " u" : (archive->do_freshen ? " f" : " a"),
+	                      archive->do_recurse ? " -r" : "",
 	                      archive->do_move ? " -d1" : "",
 	                      " -m", compression,
 	                      password_str, " -2d -i -y ",
@@ -372,8 +388,8 @@ void xa_arj_delete (XArchive *archive, GSList *file_list)
 	GString *files;
 	gchar *command;
 
-	files = xa_quote_filenames(file_list, "*?[]", FALSE);
-	command = g_strconcat(archiver[archive->type].program[0], " d -i -y ", archive->path[1], " --", files->str, NULL);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_ASTERISK);
+	command = g_strconcat(archiver[archive->type].program[0], " d", archive->do_recurse ? " -p1" : "", " -i -y ", archive->path[1], " --", files->str, NULL);
 	g_string_free(files,TRUE);
 
 	xa_run_command(archive, command);
