@@ -721,7 +721,7 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 	dialog_data = g_new0 (Multi_extract_data,1);
 	dialog_data->multi_extract = gtk_dialog_new();
 
-	gtk_window_set_position (GTK_WINDOW (dialog_data->multi_extract),GTK_WIN_POS_CENTER_ON_PARENT);
+	gtk_window_set_position (GTK_WINDOW (dialog_data->multi_extract),GTK_WIN_POS_CENTER);
 	gtk_window_set_type_hint (GTK_WINDOW (dialog_data->multi_extract),GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog_data->multi_extract),TRUE);
 	gtk_widget_set_size_request(dialog_data->multi_extract, -1, 310);
@@ -908,6 +908,20 @@ void xa_multi_extract_dialog (Multi_extract_data *dialog)
 	gint response;
 	double percent = 0.0;
 
+	if (!*gtk_entry_get_text(GTK_ENTRY(dialog->entry1)))
+	{
+		path = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(prefs_window->combo_prefered_extract_dir));
+
+		if (!path || !*path)
+		{
+			g_free(path);
+			path = g_strdup("/tmp");
+		}
+
+		gtk_entry_set_text(GTK_ENTRY(dialog->entry1), path);
+		g_free(path);
+	}
+
 	gtk_widget_show_all(dialog->multi_extract);
 run:
 	response = gtk_dialog_run(GTK_DIALOG(dialog->multi_extract));
