@@ -48,6 +48,7 @@
 #define compress (archive->type == XARCHIVETYPE_COMPRESS)
 #define lrzip    (archive->type == XARCHIVETYPE_LRZIP)
 #define lz4      (archive->type == XARCHIVETYPE_LZ4)
+#define rzip     (archive->type == XARCHIVETYPE_RZIP)
 #define xz       (archive->type == XARCHIVETYPE_XZ)
 #define zstd     (archive->type == XARCHIVETYPE_ZSTD)
 
@@ -144,9 +145,21 @@ gchar *xa_gzip_et_al_check_zstd (const gchar *compressor, const gchar *decompres
 
 static gchar *xa_gzip_et_al_password_str (const gchar *password, XArchiveType type)
 {
+	gchar *escaped, *quoted, *password_str;
+
 	if (password)
+	{
 		if (type == XARCHIVETYPE_LRZIP)
-			return g_strconcat(" --encrypt=", password, NULL);
+		{
+			escaped = xa_escape_bad_chars(password, "\"");
+			quoted = g_shell_quote(escaped);
+			password_str = g_strconcat(" --encrypt=", quoted, NULL);
+			g_free(quoted);
+			g_free(escaped);
+
+			return password_str;
+		}
+	}
 
 	return g_strdup("");
 }
@@ -158,7 +171,7 @@ gchar *xa_gzip_et_al_get_command (const gchar *program, gchar *workfile, gchar *
 	password_str = xa_gzip_et_al_password_str(password, type);
 	workfile = xa_escape_bad_chars(workfile, "\"");
 	archive = xa_quote_shell_command(archive, TRUE);
-	command = g_strconcat("sh -c \"exec ", program, " ", workfile, password_str, type == XARCHIVETYPE_LRZIP ? " -fo " : " -c > ", archive, "\"", NULL);
+	command = g_strconcat("sh -c \"exec ", program, " ", workfile, password_str, type == XARCHIVETYPE_LRZIP ? " -fo " : (type == XARCHIVETYPE_RZIP ? " -kfo " : " -c > "), archive, "\"", NULL);
 	g_free(archive);
 	g_free(workfile);
 	g_free(password_str);
@@ -177,6 +190,7 @@ static compressor_t xa_gzip_et_al_compressor (XArchive *archive)
 	compressor_t lz4_compressor = {FALSE, 1, 1, 9, 1};
 	compressor_t lzma_compressor = {FALSE, 1, 7, 9, 1};
 	compressor_t lzop_compressor = {FALSE, 1, 3, 9, 1};
+	compressor_t rzip_compressor = {TRUE, 1, 6, 9, 1};
 	compressor_t xz_compressor = {FALSE, 0, 6, 9, 1};
 	compressor_t zstd_compressor = {FALSE, 1, 3, 19, 1};
 	compressor_t void_compressor = {FALSE, 0, 0, 0, 0};
@@ -214,6 +228,9 @@ static compressor_t xa_gzip_et_al_compressor (XArchive *archive)
 		case XARCHIVETYPE_LZOP:
 			return lzop_compressor;
 
+		case XARCHIVETYPE_RZIP:
+			return rzip_compressor;
+
 		case XARCHIVETYPE_ZSTD:
 			return zstd_compressor;
 
@@ -224,7 +241,7 @@ static compressor_t xa_gzip_et_al_compressor (XArchive *archive)
 
 static void xa_gzip_et_al_can (XArchive *archive, gboolean can)
 {
-	archive->can_test = (can && !bzip && !compress && (!zstd || zstd_can_test));
+	archive->can_test = (can && !bzip && !compress && !rzip && (!zstd || zstd_can_test));
 	archive->can_extract = can;
 	archive->can_password = (can && lrzip && lrzip_can_password);
 	archive->can_overwrite = can;
@@ -619,7 +636,7 @@ void xa_gzip_et_al_list (XArchive *archive)
 		}
 	}
 
-	if (archive->type == XARCHIVETYPE_LZ4 && (archive->tag & 0xff) == 'm')
+	if (archive->type == XARCHIVETYPE_LZ4 && TAGTYPE(archive->tag) == 'm')
 	{
 		int skip, offset;
 		uint32_t uncompressed, blocksize;
@@ -726,7 +743,7 @@ void xa_gzip_et_al_list (XArchive *archive)
 	archive_path = xa_quote_shell_command(archive->path[2], TRUE);
 
 	archive->child_dir = g_strdup(archive->working_dir);
-	command = g_strconcat("sh -c \"exec ", archiver[archive->type].program[0], " -d", password_str, " ", archive_path, lrzip ? " -fo " : " -c > ", decompfile, "\"", NULL);
+	command = g_strconcat("sh -c \"exec ", archiver[archive->type].program[0], " -d", password_str, " ", archive_path, lrzip ? " -fo " : (rzip ? " -kfo " : " -c > "), decompfile, "\"", NULL);
 	xa_run_command(archive, command);
 	g_free(command);
 
@@ -874,6 +891,7 @@ void xa_gzip_et_al_list (XArchive *archive)
 		case XARCHIVETYPE_COMPRESS:
 		case XARCHIVETYPE_LZ4:
 		case XARCHIVETYPE_LZMA:
+		case XARCHIVETYPE_RZIP:
 			break;
 
 		default:
@@ -936,7 +954,7 @@ gboolean xa_gzip_et_al_extract (XArchive *archive, GSList *file_list)
 	gchar *command, *filename, *files_str, *out_dir, *out_file, *archive_path, *extraction_dir, *password_str;
 	gboolean result;
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
 
 	if (*files->str)
 	{
@@ -965,7 +983,7 @@ gboolean xa_gzip_et_al_extract (XArchive *archive, GSList *file_list)
 	if (archive->do_overwrite || !g_file_test(out_file, G_FILE_TEST_EXISTS))
 	{
 		password_str = xa_gzip_et_al_password_str(archive->password, archive->type);
-		command = g_strconcat("sh -c \"exec ", archiver[archive->type].program[0], " -d", password_str, " ", archive_path, lrzip ? " -fo " : " -c > ", extraction_dir, "/", files_str, "\"", NULL);
+		command = g_strconcat("sh -c \"exec ", archiver[archive->type].program[0], " -d", password_str, " ", archive_path, lrzip ? " -fo " : (rzip ? " -kfo " : " -c > "), extraction_dir, "/", files_str, "\"", NULL);
 		g_free(password_str);
 	}
 	else
@@ -993,7 +1011,7 @@ void xa_gzip_et_al_add (XArchive *archive, GSList *file_list)
 
 	compression = g_strdup_printf("%hu", archive->compression);
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
 	files_str = xa_escape_bad_chars(files->str, "\"");
 	archive_path = xa_quote_shell_command(archive->path[2] ? archive->path[2] : archive->path[0], TRUE);
 
@@ -1004,8 +1022,8 @@ void xa_gzip_et_al_add (XArchive *archive, GSList *file_list)
 
 	password_str = xa_gzip_et_al_password_str(archive->password, archive->type);
 
-	if (lrzip)
-		out = g_strconcat(" -fo ", archive_path, " --", files_str, NULL);
+	if (lrzip || rzip)
+		out = g_strconcat(lrzip ? " -fo " : " -kfo ", archive_path, " --", files_str, NULL);
 	else
 		out = g_strconcat(" -c", bzip ? "" : " --", files_str, " > ", archive_path, NULL);
 
