@@ -24,6 +24,7 @@
 #include "add_dialog.h"
 #include "main.h"
 #include "pref_dialog.h"
+#include "rar.h"
 #include "string_utils.h"
 #include "support.h"
 #include "window.h"
@@ -251,6 +252,7 @@ static void xa_dir_sidebar_drag_data_received (GtkWidget *widget, GdkDragContext
 	GtkTreeModel *model;
 	GtkTreeIter iter, parent;
 	GString *pathname;
+	gboolean recurse;
 
 	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
 	path = g_object_get_data(G_OBJECT(context), "tree_path");
@@ -315,10 +317,19 @@ failed:
 	archive[idx]->do_full_path = FALSE;
 	archive[idx]->do_update = FALSE;
 	archive[idx]->do_freshen = FALSE;
+	archive[idx]->do_recurse = FALSE;
 	archive[idx]->do_move = FALSE;
 	archive[idx]->do_solid = FALSE;
 
-	xa_execute_add_commands(archive[idx], list, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir)), TRUE);
+	recurse = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir));
+
+	if (archive[idx]->can_recurse[1])
+	{
+		archive[idx]->do_recurse = (recurse || (archive[idx]->can_recurse[1] == FORCED));
+		recurse = FALSE;
+	}
+
+	xa_execute_add_commands(archive[idx], list, recurse, !archive[idx]->do_recurse);
 
 	g_string_free(pathname, TRUE);
 	g_slist_free_full(list, g_free);
@@ -374,6 +385,52 @@ static gboolean xa_dir_sidebar_drag_motion (GtkWidget *widget, GdkDragContext *c
 	return TRUE;
 }
 
+static gboolean xa_dir_sidebar_key_pressed (GtkWidget *widget, GdkEventKey *event)
+{
+	guint key;
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+
+	key = event->keyval;
+
+	if (key == GDK_KEY_Left || key == GDK_KEY_Right)
+	{
+		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(archive_dir_treeview));
+
+		if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+			return FALSE;
+
+		path = gtk_tree_model_get_path(model, &iter);
+
+		switch (key)
+		{
+			case GDK_KEY_Left:
+				if (gtk_tree_view_row_expanded(GTK_TREE_VIEW(archive_dir_treeview), path))
+				{
+					gtk_tree_view_collapse_row(GTK_TREE_VIEW(archive_dir_treeview), path);
+					gtk_tree_path_free(path);
+					return TRUE;
+				}
+				break;
+
+			case GDK_KEY_Right:
+				if (!gtk_tree_view_row_expanded(GTK_TREE_VIEW(archive_dir_treeview), path))
+				{
+					gtk_tree_view_expand_row(GTK_TREE_VIEW(archive_dir_treeview), path, FALSE);
+					gtk_tree_path_free(path);
+					return TRUE;
+				}
+				break;
+		}
+
+		gtk_tree_path_free(path);
+	}
+
+	return FALSE;
+}
+
 static void xa_restore_navigation (int idx)
 {
 	gboolean back = FALSE,up = FALSE,forward = FALSE,home=FALSE;
@@ -466,7 +523,6 @@ static void xa_select_by_pattern_dialog (GtkMenuItem *menuitem, gpointer user_da
 
 	ddialog1 = gtk_dialog_new ();
 	gtk_window_set_title (GTK_WINDOW (ddialog1),_("Select by Pattern"));
-	gtk_window_set_modal (GTK_WINDOW (ddialog1),TRUE);
 	gtk_window_set_type_hint (GTK_WINDOW (ddialog1),GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_widget_set_size_request(ddialog1, 264, -1);
 	gtk_window_set_transient_for (GTK_WINDOW (ddialog1),GTK_WINDOW (xa_main_window));
@@ -992,6 +1048,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	gtk_tool_item_set_homogeneous (GTK_TOOL_ITEM (back_button),FALSE);
 	gtk_container_add (GTK_CONTAINER (toolbar1),back_button);
 	gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(back_button), _("Back"));
+	gtk_widget_add_accelerator(back_button, "clicked", accel_group, GDK_KEY_Left, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
 
 	tmp_image = gtk_image_new_from_stock ("gtk-go-up",tmp_toolbar_icon_size);
 	gtk_widget_show (tmp_image);
@@ -1001,6 +1058,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	gtk_tool_item_set_homogeneous (GTK_TOOL_ITEM (up_button),FALSE);
 	gtk_container_add (GTK_CONTAINER (toolbar1),up_button);
 	gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(up_button), _("Up"));
+	gtk_widget_add_accelerator(up_button, "clicked", accel_group, GDK_KEY_Up, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
 
 	tmp_image = gtk_image_new_from_stock ("gtk-go-forward",tmp_toolbar_icon_size);
 	gtk_widget_show (tmp_image);
@@ -1010,6 +1068,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	gtk_tool_item_set_homogeneous (GTK_TOOL_ITEM (forward_button),FALSE);
 	gtk_container_add (GTK_CONTAINER (toolbar1),forward_button);
 	gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(forward_button), _("Forward"));
+	gtk_widget_add_accelerator(forward_button, "clicked", accel_group, GDK_KEY_Right, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
 
 	tmp_image = gtk_image_new_from_stock ("gtk-goto-top",tmp_toolbar_icon_size);
 	gtk_widget_show (tmp_image);
@@ -1019,6 +1078,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	gtk_tool_item_set_homogeneous (GTK_TOOL_ITEM (home_button),FALSE);
 	gtk_container_add (GTK_CONTAINER (toolbar1),home_button);
 	gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(home_button), _("Root"));
+	gtk_widget_add_accelerator(home_button, "clicked", accel_group, GDK_KEY_Home, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
 
 	separatortoolitem3 = (GtkWidget*) gtk_separator_tool_item_new ();
 	gtk_widget_show (separatortoolitem3);
@@ -1098,6 +1158,7 @@ void xa_create_main_window (GtkWidget *xa_main_window, gboolean show_location, g
 	g_signal_connect(G_OBJECT(archive_dir_treeview), "row-expanded", G_CALLBACK(xa_dir_sidebar_row_expanded), archive_dir_treestore);
 	g_signal_connect(G_OBJECT(archive_dir_treeview), "drag-data-received", G_CALLBACK(xa_dir_sidebar_drag_data_received), NULL);
 	g_signal_connect(G_OBJECT(archive_dir_treeview), "drag-motion", G_CALLBACK(xa_dir_sidebar_drag_motion), NULL);
+	g_signal_connect(G_OBJECT(archive_dir_treeview), "key-press-event", G_CALLBACK(xa_dir_sidebar_key_pressed), NULL);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW (archive_dir_treeview));
 	changed = g_signal_connect(G_OBJECT(sel), "changed", G_CALLBACK(xa_dir_sidebar_row_selected), NULL);
 
@@ -1768,10 +1829,30 @@ void xa_combo_box_text_append_compressor_types (GtkComboBoxText *combo_box_text)
 			while (list)
 			{
 				if (list->data)
+				{
 					/* only zip can create epub compliant with the standards */
-					if (strcmp(list->data, "epub") != 0 || archiver[i].ask == xa_zip_ask)
-						sorted = g_slist_append(sorted, list->data);
+					if (strcmp(list->data, "epub") == 0 && archiver[i].ask != xa_zip_ask)
+						goto next;
 
+					/* rar program version 7 cannot create rar v4 type archives */
+					if (archiver[i].ask == xa_rar_ask && rar_version >= 7)
+					{
+						gint pos;
+						gushort tag = 4;
+
+						pos = g_slist_index(archiver[i].tags, list->data);
+
+						if (pos > 0)
+							tag = (gushort) GPOINTER_TO_UINT(g_slist_nth_data(archiver[i].tags, pos - 1));
+
+						if (TAGTYPE(tag) == 4)
+							goto next;
+					}
+
+					sorted = g_slist_prepend(sorted, list->data);
+				}
+
+next:
 				list = list->next;
 			}
 		}
