@@ -44,16 +44,21 @@ gboolean xa_lha_check_program (gchar *path)
 
 void xa_lha_ask (XArchive *archive)
 {
+	gboolean read_only;
 	compressor_t lha_compressor = {FALSE, 5, 5, 7, 1};
 
+	read_only = (archive->tag == 'x');   // exe
+
 	archive->can_test = TRUE;
 	archive->can_extract = TRUE;
-	archive->can_add = archiver[archive->type].is_compressor;
-	archive->can_delete = archiver[archive->type].is_compressor;
+	archive->can_add = (archiver[archive->type].is_compressor && !read_only);
+	archive->can_delete = (archiver[archive->type].is_compressor && !read_only);
 	archive->can_full_path[0] = TRUE;
 	archive->can_full_path[1] = archiver[archive->type].is_compressor;   // n.b.: adds leading slash
 	archive->can_overwrite = TRUE;
 	archive->can_update[1] = archiver[archive->type].is_compressor;
+	archive->can_recurse[0] = TRUE;
+	archive->can_recurse[1] = (archiver[archive->type].is_compressor ? FORCED : FALSE);
 	archive->can_move = archiver[archive->type].is_compressor;
 	archive->can_compress = archiver[archive->type].is_compressor;
 	archive->compressor = lha_compressor;
@@ -197,7 +202,7 @@ gboolean xa_lha_extract (XArchive *archive, GSList *file_list)
 	gchar *command;
 	gboolean result;
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_ASTERISK);
 	command = g_strconcat(archiver[archive->type].program[0],
 	                      archive->do_full_path ? " x" : " xi",
 	                      archive->do_overwrite ? "f" : "",
@@ -218,7 +223,7 @@ void xa_lha_add (XArchive *archive, GSList *file_list)
 
 	compression = g_strdup_printf("%hu", archive->compression);
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
 	command = g_strconcat(archiver[archive->type].program[0],
 	                      archive->do_update ? " u" : " a",
 	                      archive->do_move ? "d" : "",
@@ -236,7 +241,7 @@ void xa_lha_delete (XArchive *archive, GSList *file_list)
 	GString *files;
 	gchar *command;
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_ASTERISK);
 	command = g_strconcat(archiver[archive->type].program[0], " d ", archive->path[1], files->str, NULL);
 	g_string_free(files,TRUE);
 
