@@ -90,7 +90,7 @@ static GOptionEntry entries[] =
 		NULL
 	},
 	{	"info", 'i', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_info,
-		N_("Show found command line programs to be\n                                     used and exit"),
+		N_("Show found command-line programs to be\n                                     used and exit"),
 		NULL
 	},
 	{	"version", 'v', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_version,
@@ -104,7 +104,7 @@ static void xa_check_available_archivers ()
 	gchar *path, *cpio, *lbzip2, *lsar, *pbzip2, *pigz, *plzip, *sevenz, *unar, *xz, *zstd;
 	gboolean is7z = TRUE, is7za = TRUE, is7zr = TRUE, standard;
 
-	/* (un)compressors that can handle various types */
+	/* (de)compressors that can handle various types */
 
 	sevenz = g_find_program_in_path("7zz");
 
@@ -149,7 +149,7 @@ static void xa_check_available_archivers ()
 			/* alternative ... */
 			path = g_strdup(lsar);
 		if (unar)
-			/* ... uncompressor */
+			/* ... decompressor */
 			archiver[type].program[1] = g_strdup(unar);
 	}
 
@@ -159,8 +159,13 @@ static void xa_check_available_archivers ()
 		archiver[type].is_compressor = standard;
 		archiver[type].type = g_slist_append(archiver[type].type, "7zip");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.7z");
+		/* comic book */
+		archiver[type].type = g_slist_append(archiver[type].type, "cb7");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.cb7");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('c'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 		/* self-extracting Nullsoft Installer */
-		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('n'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER(TAG('x','n')));
 		archiver[type].tags = g_slist_append(archiver[type].tags, _("Nullsoft Installer"));
 
 		archiver[type].ask = (standard ? xa_7zip_ask : xa_unar_ask);
@@ -184,11 +189,11 @@ static void xa_check_available_archivers ()
 		{
 			/* alternative ... */
 			path = g_strdup(lsar);
-			/* ... uncompressor */
+			/* ... decompressor */
 			archiver[type].program[1] = g_strdup(unar);
 		}
 		else if (is7z)
-			/* alternative uncompressor */
+			/* alternative decompressor */
 			path = g_strconcat(sevenz, " -tar", NULL);
 	}
 
@@ -223,7 +228,7 @@ static void xa_check_available_archivers ()
 	if (!standard)
 	{
 		if (is7z)
-			/* alternative uncompressor */
+			/* alternative decompressor */
 			path = g_strconcat(sevenz, " -tarj", NULL);
 		else
 		{
@@ -231,7 +236,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 	}
@@ -315,7 +320,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 		else
@@ -375,6 +380,23 @@ static void xa_check_available_archivers ()
 		archiver[type].extract = xa_7zip_extract;
 	}
 
+	/* chm */
+
+	type = XARCHIVETYPE_CHM;
+	path = (is7z ? g_strconcat(sevenz, " -tchm", NULL) : NULL);
+
+	if (path)
+	{
+		archiver[type].program[0] = path;
+		archiver[type].type = g_slist_append(archiver[type].type, "chm");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.chm");
+
+		archiver[type].ask = xa_7zip_ask;
+		archiver[type].list = xa_7zip_list;
+		archiver[type].test = xa_7zip_test;
+		archiver[type].extract = xa_7zip_extract;
+	}
+
 	/* compress */
 
 	type = XARCHIVETYPE_COMPRESS;
@@ -390,7 +412,7 @@ static void xa_check_available_archivers ()
 	if (!standard)
 	{
 		if (is7za)
-			/* alternative uncompressor */
+			/* alternative decompressor */
 			path = g_strconcat(sevenz, " -tZ", NULL);
 		else
 		{
@@ -398,7 +420,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 	}
@@ -421,12 +443,12 @@ static void xa_check_available_archivers ()
 	type = XARCHIVETYPE_CPIO;
 	path = cpio;
 
-	standard = (path != NULL);
+	standard = (path && xa_cpio_check_version(path));
 
 	if (!standard)
 	{
 		if (is7z)
-			/* alternative uncompressor */
+			/* alternative decompressor */
 			path = g_strconcat(sevenz, " -tcpio", NULL);
 		else
 		{
@@ -434,7 +456,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 	}
@@ -488,7 +510,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 		else
@@ -535,7 +557,7 @@ static void xa_check_available_archivers ()
 	if (!standard)
 	{
 		if (is7z)
-			/* alternative uncompressor */
+			/* alternative decompressor */
 			path = g_strconcat(sevenz, " -tlzh", NULL);
 		else
 		{
@@ -543,7 +565,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 	}
@@ -606,7 +628,7 @@ static void xa_check_available_archivers ()
 		/* Mozilla's proprietary LZ4 Frame Formats */
 		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('m'));
 		archiver[type].tags = g_slist_append(archiver[type].tags, "mozlz4");
-		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('m' + 0x100));
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER(TAG('m','j')));
 		archiver[type].tags = g_slist_append(archiver[type].tags, "mozJSSClz4");
 
 		archiver[type].ask = xa_gzip_et_al_ask;
@@ -664,7 +686,7 @@ static void xa_check_available_archivers ()
 	if (!standard)
 	{
 		if (is7zr)
-			/* alternative uncompressor */
+			/* alternative decompressor */
 			path = g_strconcat(sevenz, " -tlzma", NULL);
 		else
 		{
@@ -672,7 +694,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 	}
@@ -741,7 +763,7 @@ static void xa_check_available_archivers ()
 	{
 		if (is7z)
 		{
-			/* alternative uncompressor */
+			/* alternative decompressor */
 			path = g_strconcat(sevenz, " -trar", NULL);
 			archiver[type].program[1] = g_strconcat(sevenz, " -trar5", NULL);
 		}
@@ -751,7 +773,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 	}
@@ -759,15 +781,27 @@ static void xa_check_available_archivers ()
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].type = g_slist_append(archiver[type].type, "rar");
-		archiver[type].glob = g_slist_append(archiver[type].glob, "*.rar");
+		archiver[type].type = g_slist_append(archiver[type].type, "rar4");
+		archiver[type].glob = g_slist_append(archiver[type].glob, " .rar");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER(4));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
+		/* comic book */
+		archiver[type].type = g_slist_append(archiver[type].type, "cbr4");
+		archiver[type].glob = g_slist_append(archiver[type].glob, " .cbr");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER(TAG(4,'c')));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 
 		if (!standard || (rar_version >= 5))
 		{
-			archiver[type].type = g_slist_append(archiver[type].type, "rar5");
-			archiver[type].glob = g_slist_append(archiver[type].glob, " .rar");
+			archiver[type].type = g_slist_append(archiver[type].type, "rar");
+			archiver[type].glob = g_slist_append(archiver[type].glob, "*.rar");
 			archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER(5));
 			archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
+			/* comic book */
+			archiver[type].type = g_slist_append(archiver[type].type, "cbr");
+			archiver[type].glob = g_slist_append(archiver[type].glob, "*.cbr");
+			archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER(TAG(5,'c')));
+			archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 		}
 
 		archiver[type].ask = FUNC(standard, xa_rar_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
@@ -788,7 +822,7 @@ static void xa_check_available_archivers ()
 	if (!standard)
 	{
 		if (is7z)
-			/* alternative uncompressor */
+			/* alternative decompressor */
 			path = g_strconcat(sevenz, " -trpm", NULL);
 		else
 		{
@@ -796,7 +830,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 	}
@@ -813,6 +847,24 @@ static void xa_check_available_archivers ()
 		archiver[type].extract = FUNC(standard, xa_rpm_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
 	}
 
+	/* rzip */
+
+	type = XARCHIVETYPE_RZIP;
+	path = g_find_program_in_path("rzip");
+
+	if (path)
+	{
+		archiver[type].program[0] = path;
+		archiver[type].is_compressor = TRUE;
+		archiver[type].type = g_slist_append(archiver[type].type, "rzip");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.rz");
+
+		archiver[type].ask = xa_gzip_et_al_ask;
+		archiver[type].list = xa_gzip_et_al_list;
+		archiver[type].extract = xa_gzip_et_al_extract;
+		archiver[type].add = xa_gzip_et_al_add;
+	}
+
 	/* SquashFS */
 
 	type = XARCHIVETYPE_SQUASHFS;
@@ -823,7 +875,7 @@ static void xa_check_available_archivers ()
 	if (!standard)
 	{
 		if (is7z)
-			/* alternative uncompressor */
+			/* alternative decompressor */
 			path = g_strconcat(sevenz, " -tsquashfs", NULL);
 	}
 	else
@@ -878,7 +930,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 	}
@@ -889,6 +941,11 @@ static void xa_check_available_archivers ()
 		archiver[type].is_compressor = (standard || is7za);
 		archiver[type].type = g_slist_append(archiver[type].type, "tar");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.tar");
+		/* comic book */
+		archiver[type].type = g_slist_append(archiver[type].type, "cbt");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.cbt");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('c'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 
 		archiver[type].ask = FUNC(standard, xa_tar_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
 		archiver[type].list = FUNC(standard, xa_tar_list, is7za, xa_7zip_list, lsar, xa_unar_list);
@@ -925,7 +982,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 		else
@@ -985,7 +1042,7 @@ static void xa_check_available_archivers ()
 				/* alternative ... */
 				path = g_strdup(lsar);
 			if (unar)
-				/* ... uncompressor */
+				/* ... decompressor */
 				archiver[type].program[1] = g_strdup(unar);
 		}
 		else
@@ -1007,25 +1064,40 @@ static void xa_check_available_archivers ()
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.cbz");
 		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('c'));
 		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
+		/* Office Open XML document */
+		archiver[type].type = g_slist_append(archiver[type].type, "docx");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.docx");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER(TAG('w','d')));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 		/* electronic publication */
 		archiver[type].type = g_slist_append(archiver[type].type, "epub");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.epub");
 		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('e'));
 		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
+		/* FictionBook */
+		archiver[type].type = g_slist_append(archiver[type].type, "fbz");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.fbz");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('f'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 		/* java archive */
 		archiver[type].type = g_slist_append(archiver[type].type, "jar");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.jar");
 		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('j'));
 		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
+		/* OpenDocument Format text */
+		archiver[type].type = g_slist_append(archiver[type].type, "odt");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.odt");
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER(TAG('o','t')));
+		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 		/* OpenOffice.org extension */
 		archiver[type].type = g_slist_append(archiver[type].type, "oxt");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.oxt");
-		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('o'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('t'));
 		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 		/* XPInstall */
 		archiver[type].type = g_slist_append(archiver[type].type, "xpi");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.xpi");
-		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('x'));
+		archiver[type].tags = g_slist_append(archiver[type].tags, GUINT_TO_POINTER('p'));
 		archiver[type].tags = g_slist_append(archiver[type].tags, g_slist_last(archiver[type].type)->data);
 
 		archiver[type].ask = FUNC(standard, xa_zip_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
@@ -1096,6 +1168,7 @@ static void xa_check_available_archivers ()
 			{XARCHIVETYPE_LZIP, {"tar.lzip", NULL}, {"*.tar.lz", ""}},
 			{XARCHIVETYPE_LZMA, {"tar.lzma", NULL}, {"*.tar.lzma", "*.tlz"}},
 			{XARCHIVETYPE_LZOP, {"tar.lzop", NULL}, {"*.tar.lzo", "*.tzo"}},
+			{XARCHIVETYPE_RZIP, {"tar.rzip", NULL}, {"*.tar.rz", ""}},
 			{XARCHIVETYPE_XZ, {"tar.xz", NULL}, {"*.tar.xz", "*.txz"}},
 			{XARCHIVETYPE_ZSTD, {"tar.zstd", NULL}, {"*.tar.zst", "*.tzst"}},
 			{XARCHIVETYPE_UNKNOWN, {NULL, NULL}, {NULL, NULL}}
@@ -1181,9 +1254,9 @@ static XArchive *xa_init_structure_from_cmd_line (char *filename)
 static void xa_cmd_line_error (const char *argv1, const gchar *header)
 {
 	if (argv1)
-		xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, header, _("Can't open the archive!\n"));
+		xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, header, _("Can't open the archive!"));
 	else
-		xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, header, _("You missed the archive name!\n"));
+		xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, header, _("You missed the archive name!"));
 }
 
 int main (int argc, char **argv)
@@ -1216,18 +1289,20 @@ int main (int argc, char **argv)
 
 	if ( cli_error != NULL )
 	{
-		g_print (_("xarchiver: %s\nTry xarchiver --help to see a full list of available command line options.\n"),cli_error->message);
+		g_print (_("xarchiver: %s\nTry xarchiver --help to see a full list of available command-line options.\n"),cli_error->message);
 		g_error_free (cli_error);
 		return 0;
 	}
     /* print version information */
     if (opt_version)
     {
-        g_print("%s %s \n\n", PACKAGE_NAME, PACKAGE_VERSION);
+        g_print("%s %s\n\n", PACKAGE_NAME, PACKAGE_VERSION);
+        g_print ("%s\n", "Copyright (c) " COPYRIGHT_YEAR);
+        g_print ("\t%s\n", COPYRIGHT_HOLDER " - Colossus <colossus73@gmail.com>");
+        g_print ("%s\n", "Copyright \xC2\xA9 " MAINTAINER_YEAR);
+        g_print ("\t%s\n\n", MAINTAINER);
         g_print (_("Maintained by "));
         g_print (MAINTAINER ".\n\n");
-        g_print ("%s\n", "Copyright (c) " COPYRIGHT_YEAR);
-        g_print ("\t%s\n\n", COPYRIGHT_HOLDER " - Colossus <colossus73@gmail.com>");
         g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
         g_print ("\n");
 
@@ -1373,7 +1448,7 @@ int main (int argc, char **argv)
 				goto leave;
 			}
 
-			archive = xa_new_archive_dialog(opt_compress, NULL);
+			archive = xa_new_archive_dialog(argc == 1 ? opt_compress : NULL, NULL);
 
 			if (archive == NULL)
 			{
@@ -1384,7 +1459,7 @@ int main (int argc, char **argv)
 			if (archive->path[0] != NULL)
 			{
 				gchar *fname;
-				gboolean is_dir;
+				gboolean is_dir, recurse;
 				GSList *files = NULL;
 
 				fname = xa_make_full_path(opt_compress);
@@ -1415,10 +1490,19 @@ int main (int argc, char **argv)
 				archive->do_full_path = FALSE;
 				archive->do_update = FALSE;
 				archive->do_freshen = FALSE;
+				archive->do_recurse = FALSE;
 				archive->do_move = FALSE;
 				archive->do_solid = FALSE;
 
-				xa_execute_add_commands(archive, files, TRUE, TRUE);
+				recurse = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir));
+
+				if (archive->can_recurse[1])
+				{
+					archive->do_recurse = (recurse || (archive->can_recurse[1] == FORCED));
+					recurse = FALSE;
+				}
+
+				xa_execute_add_commands(archive, files, recurse, !archive->do_recurse);
 			}
 		}
 		/* Switch -a */
