@@ -77,6 +77,7 @@ const char *xa_get_stock_mime_icon(const char *filename)
 	         strcmp(mime,"application/vnd.android.package-archive") == 0 ||
 	         strcmp(mime,"application/vnd.debian.binary-package") == 0 ||
 	         strcmp(mime,"application/vnd.ms-cab-compressed") == 0 ||
+	         strcmp(mime,"application/vnd.ms-htmlhelp") == 0 ||
 	         strcmp(mime,"application/vnd.openofficeorg.extension") == 0 ||
 	         strcmp(mime,"application/vnd.rar") == 0 || /* legacy */ strcmp(mime,"application/x-rar") == 0 ||
 	         strcmp(mime,"application/vnd.snap") == 0 ||
@@ -84,8 +85,12 @@ const char *xa_get_stock_mime_icon(const char *filename)
 	         strcmp(mime,"application/x-7z-compressed") == 0 ||
 	         strcmp(mime,"application/x-archive") == 0 ||
 	         strcmp(mime,"application/x-arj") == 0 ||
-	         strcmp(mime,"application/x-bzip") == 0 ||
-	         strcmp(mime,"application/x-bzip-compressed-tar") == 0 ||
+	         strcmp(mime,"application/x-bzip1") == 0 ||
+	         strcmp(mime,"application/x-bzip1-compressed-tar") == 0 ||
+	         strcmp(mime,"application/x-bzip2") == 0 || /* legacy */ strcmp(mime,"application/x-bzip") == 0 ||
+	         strcmp(mime,"application/x-bzip2-compressed-tar") == 0 || /* legacy */ strcmp(mime,"application/x-bzip-compressed-tar") == 0 ||
+	         strcmp(mime,"application/x-bzip3") == 0 ||
+	         strcmp(mime,"application/x-bzip3-compressed-tar") == 0 ||
 	         strcmp(mime,"application/x-cd-image") == 0 ||
 	         strcmp(mime,"application/x-compress") == 0 ||
 	         strcmp(mime,"application/x-compressed-tar") == 0 ||
@@ -109,6 +114,7 @@ const char *xa_get_stock_mime_icon(const char *filename)
 	         strcmp(mime,"application/x-xpinstall") == 0 ||
 	         strcmp(mime,"application/x-xz") == 0 ||
 	         strcmp(mime,"application/x-xz-compressed-tar") == 0 ||
+	         strcmp(mime,"application/x-zpaq") == 0 ||
 	         strcmp(mime,"application/x-zstd-compressed-tar") == 0 ||
 	         strcmp(mime,"application/zip") == 0 ||
 	         strcmp(mime,"application/zstd") == 0)
