@@ -162,86 +162,95 @@ static void xa_open_with_dialog_browse_custom_command (GtkButton *button, Open_w
 static void xa_parse_desktop_file (const gchar *path, const gchar *name, Open_with_data *data)
 {
 	gchar *filename, *line, *key;
-	gchar *app_name = NULL, *app_exec = NULL, *app_icon = NULL;
-	GIOStatus status;
 	GIOChannel *file;
-	gboolean group_line = FALSE, app_multiple = FALSE, has_mimetype = FALSE;
 	const gchar * const *langs, * const *l;
+	gboolean group_line = FALSE, app_multiple = FALSE, has_mimetype = FALSE;
+	gchar *app_name = NULL, *app_exec = NULL, *app_icon = NULL;
 	gint size;
 
-	filename = g_strconcat(path,"/",name,NULL);
-	file = g_io_channel_new_file(filename,"r",NULL);
+	filename = g_strconcat(path, "/", name, NULL);
+	file = g_io_channel_new_file(filename, "r", NULL);
 	g_free(filename);
-	if (file == NULL)
+
+	if (!file)
 		return;
+
+	g_io_channel_set_encoding(file, NULL, NULL);
 	langs = g_get_language_names();
-	g_io_channel_set_encoding(file,NULL,NULL);
-	do
+
+	while (g_io_channel_read_line(file, &line, NULL, NULL, NULL) == G_IO_STATUS_NORMAL)
 	{
-		status = g_io_channel_read_line (file, &line, NULL, NULL, NULL);
-		if (line != NULL)
+		if (strcmp(g_strchomp(line), "[Desktop Entry]") == 0)
 		{
-			if (strcmp(g_strchomp(line), "[Desktop Entry]") == 0)
-			{
-				group_line = TRUE;
-				continue;
-			}
-			if (group_line && *line == '[')
-				break;
-			if (!group_line)
-				continue;
-			if (g_str_has_prefix(line, "Name["))
-			{
-				l = langs;
-
-				while (*l)
-				{
-					key = g_strconcat("Name[", *l, "]=", NULL);
-
-					if (g_str_has_prefix(line, key))
-					{
-						g_free(app_name);
-						app_name = g_strndup(line + strlen(key), strlen(line) - strlen(key));
-						g_free(key);
-						break;
-					}
-
-					g_free(key);
-					l++;
-				}
-			}
-			if (!app_name && g_str_has_prefix(line, "Name="))
-			{
-				app_name = g_strndup(line + 5, strlen(line) - 5);
-				continue;
-			}
-			if (g_str_has_prefix(line,"Exec="))
-			{
-				app_exec = strstr(line, " %");
-				if (app_exec)
-				{
-					app_multiple = (app_exec[2] == 'F' || app_exec[2] == 'U');
-					app_exec = g_strndup(line + 5, app_exec - (line + 5));
-				}
-				else
-					app_exec = g_strndup(line + 5, strlen(line) - 5);
-				continue;
-			}
-			if (g_str_has_prefix(line,"Icon="))
-			{
-				app_icon = strrchr(line, '.');
-				if (app_icon)
-					app_icon = g_strndup(line + 5,app_icon - (line+5));
-				else
-					app_icon = g_strndup(line + 5, strlen(line) - 5);
-				continue;
-			}
-			if (g_str_has_prefix(line,"MimeType="))
-				has_mimetype = TRUE;
-			g_free(line);
+			group_line = TRUE;
+			continue;
 		}
+
+		if (group_line && *line == '[')
+			break;
+
+		if (!group_line)
+			continue;
+
+		if (g_str_has_prefix(line, "Name["))
+		{
+			l = langs;
+
+			while (*l)
+			{
+				key = g_strconcat("Name[", *l, "]=", NULL);
+
+				if (g_str_has_prefix(line, key))
+				{
+					g_free(app_name);
+					app_name = g_strndup(line + strlen(key), strlen(line) - strlen(key));
+					g_free(key);
+					break;
+				}
+
+				g_free(key);
+				l++;
+			}
+		}
+
+		if (!app_name && g_str_has_prefix(line, "Name="))
+		{
+			app_name = g_strndup(line + 5, strlen(line) - 5);
+			continue;
+		}
+
+		if (g_str_has_prefix(line, "Exec="))
+		{
+			app_exec = strstr(line, " %");
+
+			if (app_exec)
+			{
+				app_multiple = (app_exec[2] == 'F' || app_exec[2] == 'U');
+				app_exec = g_strndup(line + 5, app_exec - (line + 5));
+			}
+			else
+				app_exec = g_strndup(line + 5, strlen(line) - 5);
+
+			continue;
+		}
+
+		if (g_str_has_prefix(line, "Icon="))
+		{
+			app_icon = strrchr(line, '.');
+
+			if (app_icon)
+				app_icon = g_strndup(line + 5, app_icon - (line+5));
+			else
+				app_icon = g_strndup(line + 5, strlen(line) - 5);
+
+			continue;
+		}
+
+		if (g_str_has_prefix(line, "MimeType="))
+			has_mimetype = TRUE;
+
+		g_free(line);
 	}
-	while (status != G_IO_STATUS_EOF);
 
 	g_io_channel_shutdown(file, FALSE, NULL);
 
