@@ -60,12 +60,15 @@ void xa_rar_check_version (gchar *path)
 
 void xa_rar_ask (XArchive *archive)
 {
+	gboolean read_only;
 	compressor_t rar_compressor = {TRUE, 1, 3, 5, 1};
 
+	read_only = (archive->tag == 'x');   // exe
+
 	archive->can_test = TRUE;
 	archive->can_extract = TRUE;
-	archive->can_add = archiver[archive->type].is_compressor;
-	archive->can_delete = archiver[archive->type].is_compressor;
+	archive->can_add = (archiver[archive->type].is_compressor && !read_only);
+	archive->can_delete = (archiver[archive->type].is_compressor && !read_only);
 	archive->can_sfx = archiver[archive->type].is_compressor;
 	archive->can_password = archiver[archive->type].is_compressor;
 	archive->can_full_path[0] = TRUE;
@@ -76,6 +79,8 @@ void xa_rar_ask (XArchive *archive)
 	archive->can_update[1] = archiver[archive->type].is_compressor;
 	archive->can_freshen[0] = TRUE;
 	archive->can_freshen[1] = archiver[archive->type].is_compressor;
+	archive->can_recurse[0] = TRUE;
+	archive->can_recurse[1] = archiver[archive->type].is_compressor;
 	archive->can_move = archiver[archive->type].is_compressor;
 	archive->can_solid = archiver[archive->type].is_compressor;
 	archive->can_compress = archiver[archive->type].is_compressor;
@@ -85,13 +90,21 @@ void xa_rar_ask (XArchive *archive)
 
 static gchar *xa_rar_password_str (XArchive *archive)
 {
+	gchar *escaped, *password_str;
+
 	if (archive->password)
-		return g_strconcat(" -p", archive->password, NULL);
+	{
+		escaped = xa_escape_bad_chars(archive->password, ESCAPES);
+		password_str = g_strconcat(" -p", escaped, NULL);
+		g_free(escaped);
+
+		return password_str;
+	}
 	else
 		return g_strdup("");
 }
 
-static void xa_rar_parse_output (gchar *line, XArchive *archive)
+static void xa_rar_v4_parse_output (gchar *line, XArchive *archive)
 {
 	static gchar *filename;
 	static gboolean encrypted;
@@ -225,7 +238,7 @@ static void xa_rar_parse_output (gchar *line, XArchive *archive)
 	}
 }
 
-static void xa_rar5_parse_output (gchar *line, XArchive *archive)
+static void xa_rar_parse_output (gchar *line, XArchive *archive)
 {
 	XEntry *entry;
 	gpointer item[7];
@@ -432,7 +445,7 @@ void xa_rar_list (XArchive *archive)
 		const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
 		const gchar *titles[] = {_("Original Size"), _("Compressed"), _("Occupancy"), _("Date"), _("Time"), _("Attributes"), _("Checksum")};
 
-		archive->parse_output = xa_rar5_parse_output;
+		archive->parse_output = xa_rar_parse_output;
 		xa_spawn_async_process (archive,command);
 		g_free ( command );
 
@@ -450,7 +463,7 @@ void xa_rar_list (XArchive *archive)
 		const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
 		const gchar *titles[] = {_("Original Size"), _("Compressed"), _("Occupancy"), _("Date"), _("Time"), _("Attributes"), _("Checksum"), _("Method"), _("Version")};
 
-		archive->parse_output = xa_rar_parse_output;
+		archive->parse_output = xa_rar_v4_parse_output;
 		xa_spawn_async_process (archive,command);
 		g_free ( command );
 
@@ -487,7 +500,7 @@ gboolean xa_rar_extract (XArchive *archive, GSList *file_list)
 	gchar *password_str, *command;
 	gboolean result;
 
-	files = xa_quote_filenames(file_list, NULL, FALSE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITHOUT_SLASH);
 	password_str = xa_rar_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[0],
 	                      archive->do_full_path ? " x" : " e",
@@ -510,23 +523,24 @@ void xa_rar_add (XArchive *archive, GSList *file_list)
 	GString *files;
 	gchar *compression, *password_str, *command, *version_switch;
 
-	if (rar_version >= 5)
+	if (rar_version == 5 || rar_version == 6)
 	{
-		if (archive->tag == 5)
-			version_switch = " -ma5";
-		else
+		if (TAGTYPE(archive->tag) == 4)
 			version_switch = " -ma4";
+		else
+			version_switch = " -ma5";
 	}
 	else
 		version_switch = "";
 
 	compression = g_strdup_printf("%hu", archive->compression);
 
-	files = xa_quote_filenames(file_list, NULL, FALSE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITHOUT_SLASH);
 	password_str = xa_rar_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[0],
 	                      version_switch,
 	                      archive->do_update ? " u" : (archive->do_freshen ? " u -f" : " a"),
+	                      archive->do_recurse ? "" : " -r-",
 	                      archive->do_move ? " -df" : "",
 	                      archive->do_solid ? " -s" : "",
 	                      " -m", compression,
@@ -545,7 +559,7 @@ void xa_rar_delete (XArchive *archive, GSList *file_list)
 	GString *files;
 	gchar *command;
 
-	files = xa_quote_filenames(file_list, NULL, FALSE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITHOUT_SLASH);
 	command = g_strconcat(archiver[archive->type].program[0], " d -idp -y ", archive->path[1], " --", files->str, NULL);
 	g_string_free(files,TRUE);
 
