@@ -92,6 +92,9 @@ static XArchiveType xa_tar_get_compressor_type (XArchive *archive)
 		case XARCHIVETYPE_TAR_LZOP:
 			return XARCHIVETYPE_LZOP;
 
+		case XARCHIVETYPE_TAR_RZIP:
+			return XARCHIVETYPE_RZIP;
+
 		case XARCHIVETYPE_TAR_XZ:
 			return XARCHIVETYPE_XZ;
 
@@ -114,6 +117,8 @@ void xa_tar_ask (XArchive *archive)
 	archive->can_overwrite = TRUE;
 	archive->can_update[0] = TRUE;
 	archive->can_update[1] = archiver[xa_tar_get_compressor_type(archive)].is_compressor;
+	archive->can_recurse[0] = TRUE;
+	archive->can_recurse[1] = TRUE;
 	archive->can_move = archiver[xa_tar_get_compressor_type(archive)].is_compressor;
 
 	/* this is solely for display with the archive properties */
@@ -232,9 +237,9 @@ gboolean xa_tar_extract (XArchive *archive, GSList *file_list)
 			return FALSE;
 	}
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
 	command = g_strconcat(archiver[XARCHIVETYPE_TAR].program[0],
-	                      " -x --no-recursion --no-wildcards",
+	                      " -x --no-wildcards",
 	                      " -f ", archive->path[2],
 	                      archive->do_touch ? " -m" : "",
 	                      archive->do_overwrite ? "" : (archive->do_update ? " --keep-newer-files" : " -k"),
@@ -274,7 +279,7 @@ void xa_tar_add (XArchive *archive, GSList *file_list)
 	gchar *command;
 	gboolean success;
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
 
 	if (!g_file_test(archive->path[0], G_FILE_TEST_EXISTS))
 	{
@@ -296,14 +301,16 @@ void xa_tar_add (XArchive *archive, GSList *file_list)
 		}
 
 		command = g_strconcat(archiver[XARCHIVETYPE_TAR].program[0],
-		                      " -c --no-recursion --no-wildcards",
+		                      " -c --no-wildcards",
+		                      archive->do_recurse ? "" : " --no-recursion",
 		                      archive->do_move ? " --remove-files" : "",
 		                      " -f ", archive->path[2], " --", files->str, NULL);
 	}
 	else
 		command = g_strconcat(archiver[XARCHIVETYPE_TAR].program[0],
 		                      archive->do_update ? " -u" : " -r",
-		                      " --no-recursion --no-wildcards",
+		                      " --no-wildcards",
+		                      archive->do_recurse ? "" : " --no-recursion",
 		                      archive->do_move ? " --remove-files" : "",
 		                      " -f ", archive->path[2], " --", files->str, NULL);
 
@@ -334,8 +341,8 @@ void xa_tar_delete (XArchive *archive, GSList *file_list)
 	gchar *command;
 	gboolean success;
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);
-	command = g_strconcat(archiver[XARCHIVETYPE_TAR].program[0], " --delete --no-recursion --no-wildcards -f ", archive->path[2], " --", files->str, NULL);
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);
+	command = g_strconcat(archiver[XARCHIVETYPE_TAR].program[0], " --delete --no-wildcards -f ", archive->path[2], " --", files->str, NULL);
 
 	if (archive->type != XARCHIVETYPE_TAR)
 	{
