@@ -123,8 +123,6 @@ static gchar *xa_open_file_dialog ()
 		}
 
 		g_slist_free(sorted);
-
-		gtk_window_set_modal (GTK_WINDOW (File_Selector),TRUE);
 	}
 	if (open_file_filter != NULL)
 		gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (File_Selector) ,open_file_filter);
@@ -460,6 +458,7 @@ static void xa_rename_cell_edited (GtkCellRendererText *cell, const gchar *path_
 		archive->do_full_path = TRUE;
 		archive->do_update = FALSE;
 		archive->do_freshen = FALSE;
+		archive->do_recurse = FALSE;
 		archive->do_move = FALSE;
 		archive->do_solid = FALSE;
 
@@ -1681,7 +1680,7 @@ void xa_about (GtkMenuItem *menuitem,gpointer user_data)
     {
         "",
         "This version:",
-        MAINTAINER " <ib@wupperonline.de>",
+        MAINTAINER,
         "",
         "Original developer:",
         "Giuseppe Torelli <colossus73@gmail.com>",
@@ -1803,7 +1802,12 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 	}
 
 	if (memcmp(magic, "7z" "\xbc\xaf\x27\x1c", 6) == 0)
+	{
 		xa.type = XARCHIVETYPE_7ZIP;
+
+		if (g_str_has_suffix(filename, ".cb7"))
+			xa.tag = 'c';
+	}
 	else if (memcmp(magic, "!<arch>\n", 8) == 0)
 	{
 		xa.type = XARCHIVETYPE_AR;
@@ -1821,6 +1825,8 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 		xa.type = XARCHIVETYPE_BZIP3;
 	else if (memcmp(magic, "MSCF\x00\x00\x00\x00", 8) == 0)
 		xa.type = XARCHIVETYPE_CAB;
+	else if (memcmp(magic, "ITSF", 4) == 0)
+		xa.type = XARCHIVETYPE_CHM;
 	else if (memcmp(magic, "\x1f\x9d", 2) == 0)
 		xa.type = XARCHIVETYPE_COMPRESS;
 	else if (memcmp(magic, "070701", 6) == 0 ||
@@ -1853,7 +1859,7 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 		if (magic[3] == 'L')
 			xa.tag = 'm';
 		else if (magic[3] == 'J')
-			xa.tag = 'm' + 0x100;
+			xa.tag = TAG('m','j');
 	}
 	else if (memcmp(magic, "LZIP", 4) == 0)
 		xa.type = XARCHIVETYPE_LZIP;
@@ -1861,17 +1867,20 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 		xa.type = XARCHIVETYPE_LZMA;
 	else if (memcmp(magic, "\211LZO", 4) == 0)
 		xa.type = XARCHIVETYPE_LZOP;
+	else if (memcmp(magic, "RZIP", 4) == 0)
+		xa.type = XARCHIVETYPE_RZIP;
 	else if (memcmp(magic, "Rar!" "\x1a\x07\x00", 7) == 0 ||
 	         memcmp(magic, "Rar!" "\x1a\x07\x01", 7) == 0)
 	{
 		xa.type = XARCHIVETYPE_RAR;
+		xa.tag = (magic[6] == 1 ? 5 : 4);
 
-		if (magic[6] == 1)
-			xa.tag = 5;
-
-		/* a rar5 archive without a rar v5 compatible executable can't be opened */
+		/* a rar v5 type archive without a rar v5 compatible executable can't be opened */
 		if ((xa.tag == 5) && !g_slist_find(archiver[xa.type].tags, GUINT_TO_POINTER(xa.tag)))
 			archiver[xa.type].list = NULL;
+
+		if (g_str_has_suffix(filename, ".cbr"))
+			xa.tag += TAG(0,'c');
 	}
 	else if (memcmp(magic, "\xed\xab\xee\xdb", 4) == 0)
 		xa.type = XARCHIVETYPE_RPM;
@@ -1894,14 +1903,20 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 			xa.tag = 'a';
 		else if (g_str_has_suffix(filename, ".cbz"))
 			xa.tag = 'c';
+		else if (g_str_has_suffix(filename, ".docx"))
+			xa.tag = TAG('w','d');
 		else if (g_str_has_suffix(filename, ".epub"))
 			xa.tag = 'e';
+		else if (g_str_has_suffix(filename, ".fbz"))
+			xa.tag = 'f';
 		else if (g_str_has_suffix(filename, ".jar"))
 			xa.tag = 'j';
+		else if (g_str_has_suffix(filename, ".odt"))
+			xa.tag = TAG('o','t');
 		else if (g_str_has_suffix(filename, ".oxt"))
-			xa.tag = 'o';
+			xa.tag = 't';
 		else if (g_str_has_suffix(filename, ".xpi"))
-			xa.tag = 'x';
+			xa.tag = 'p';
 	}
 	else if (g_str_has_suffix(filename, ".exe") && (memcmp(magic, "MZ", 2) == 0))
 		xa = exetype(file);
@@ -1916,8 +1931,13 @@ ArchiveType xa_detect_archive_type (const gchar *filename)
 		xa.type = XARCHIVETYPE_ISO;
 	/* partly heuristic methods must come last */
 	else if (isTar(file))
+	{
 		xa.type = XARCHIVETYPE_TAR;
 
+		if (g_str_has_suffix(filename, ".cbt"))
+			xa.tag = 'c';
+	}
+
 	fclose(file);
 
 	return xa;
@@ -2285,7 +2305,7 @@ void xa_page_drag_data_received (GtkWidget *widget, GdkDragContext *context, gin
 	GSList *list = NULL, *flist;
 	ArchiveType xa;
 	gint idx;
-	gboolean archives = FALSE, files = FALSE;
+	gboolean archives = FALSE, files = FALSE, recurse;
 
 	uris = gtk_selection_data_get_uris(data);
 
@@ -2379,10 +2399,19 @@ failed:
 		archive[idx]->do_full_path = FALSE;
 		archive[idx]->do_update = FALSE;
 		archive[idx]->do_freshen = FALSE;
+		archive[idx]->do_recurse = FALSE;
 		archive[idx]->do_move = FALSE;
 		archive[idx]->do_solid = FALSE;
 
-		xa_execute_add_commands(archive[idx], list, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir)), TRUE);
+		recurse = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir));
+
+		if (archive[idx]->can_recurse[1])
+		{
+			archive[idx]->do_recurse = (recurse || (archive[idx]->can_recurse[1] == FORCED));
+			recurse = FALSE;
+		}
+
+		xa_execute_add_commands(archive[idx], list, recurse, !archive[idx]->do_recurse);
 	}
 	else
 	{
@@ -2405,8 +2434,15 @@ void xa_concat_selected_filenames (GtkTreeModel *model,GtkTreePath *treepath,Gtk
 	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
 
 	gtk_tree_model_get(model, iter, archive[idx]->columns - 1, &entry, -1);
+
 	if (entry->is_dir)
-		xa_fill_list_with_recursed_entries(entry->child,data);
+	{
+		if (archive[idx]->can_recurse[0])
+			archive[idx]->do_recurse = TRUE;
+		else
+			xa_fill_list_with_recursed_entries(entry->child, data);
+	}
+
 	filename = xa_build_full_path_name_from_entry(entry);
 	*data = g_slist_prepend (*data,filename);
 }
@@ -2698,7 +2734,7 @@ gboolean xa_treeview_mouse_button_press (GtkWidget *widget, GdkEventButton *even
 			gtk_widget_set_sensitive(copy, archive->can_extract);
 			gtk_widget_set_sensitive(paste, (selected == 1) && pasteable && archive->can_add);
 			gtk_widget_set_sensitive(ddelete, archive->can_delete);
-			gtk_widget_set_sensitive(edit, (selected == 1) && !entry->is_dir && archive->can_extract);
+			gtk_widget_set_sensitive(edit, (selected == 1) && !entry->is_dir && archive->can_extract && archive->can_add);
 			gtk_widget_set_sensitive(replace, (selected == 1) && replaceable && archive->can_add);
 			gtk_menu_popup(GTK_MENU(xa_popup_menu), NULL, NULL, NULL, xa_main_window, event->button, event->time);
 		}
@@ -2766,6 +2802,7 @@ void xa_clipboard_edit (GtkMenuItem *item, gpointer user_data)
 void xa_clipboard_paste (GtkMenuItem *item, gpointer user_data)
 {
 	gint idx;
+	gboolean recurse;
 
 	idx = xa_find_archive_index(gtk_notebook_get_current_page(notebook));
 
@@ -2785,10 +2822,19 @@ void xa_clipboard_paste (GtkMenuItem *item, gpointer user_data)
 	archive[idx]->do_full_path = FALSE;
 	archive[idx]->do_update = (XA_Clipboard.mode == XA_CLIPBOARD_EDIT);
 	archive[idx]->do_freshen = FALSE;
+	archive[idx]->do_recurse = FALSE;
 	archive[idx]->do_move = FALSE;
 	archive[idx]->do_solid = FALSE;
 
-	xa_execute_add_commands(archive[idx], XA_Clipboard.paths, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir)), TRUE);
+	recurse = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->allow_sub_dir));
+
+	if (archive[idx]->can_recurse[1])
+	{
+		archive[idx]->do_recurse = (recurse || (archive[idx]->can_recurse[1] == FORCED));
+		recurse = FALSE;
+	}
+
+	xa_execute_add_commands(archive[idx], XA_Clipboard.paths, recurse, !archive[idx]->do_recurse);
 
 	if (archive[idx]->status == XARCHIVESTATUS_ERROR)
 		return;
