@@ -29,15 +29,18 @@
 
 void xa_zip_ask (XArchive *archive)
 {
+	gboolean read_only;
 	gchar *sfx;
 	compressor_t zip_compressor = {TRUE, 1, 6, 9, 1};
 
+	read_only = (archive->tag == 'a' || archive->tag == 'x');   // apk, exe
+
 	sfx = g_find_program_in_path("unzipsfx");
 
 	archive->can_test = TRUE;
 	archive->can_extract = TRUE;
-	archive->can_add = archiver[archive->type].is_compressor;
-	archive->can_delete = archiver[archive->type].is_compressor;
+	archive->can_add = (archiver[archive->type].is_compressor && !read_only);
+	archive->can_delete = (archiver[archive->type].is_compressor && !read_only);
 	archive->can_sfx = (sfx && archiver[archive->type].is_compressor);
 	archive->can_password = TRUE;
 	archive->can_full_path[0] = TRUE;
@@ -48,6 +51,8 @@ void xa_zip_ask (XArchive *archive)
 	archive->can_update[1] = archiver[archive->type].is_compressor;
 	archive->can_freshen[0] = TRUE;
 	archive->can_freshen[1] = archiver[archive->type].is_compressor;
+	archive->can_recurse[0] = TRUE;
+	archive->can_recurse[1] = archiver[archive->type].is_compressor;
 	archive->can_move = archiver[archive->type].is_compressor;
 	archive->can_compress = archiver[archive->type].is_compressor;
 	archive->compressor = zip_compressor;
@@ -58,8 +63,16 @@ void xa_zip_ask (XArchive *archive)
 
 static gchar *xa_zip_password_str (XArchive *archive)
 {
+	gchar *escaped, *password_str;
+
 	if (archive->password)
-		return g_strconcat(" -P", archive->password, NULL);
+	{
+		escaped = xa_escape_bad_chars(archive->password, ESCAPES);
+		password_str = g_strconcat(" -P", escaped, NULL);
+		g_free(escaped);
+
+		return password_str;
+	}
 	else
 		return g_strdup("");
 }
@@ -74,7 +87,7 @@ static void xa_zip_parse_output (gchar *line, XArchive *archive)
 
 	USE_PARSER;
 
-	if (line[0] != '-' && line[0] != 'd' && line[0] != 'l')
+	if (line[0] != '-' && line[0] != 'd' && line[0] != 'l' && line[0] != '?')
 	{
 		/*
 		 * Unzip can compensate a central directory length reported as too long,
@@ -191,7 +204,7 @@ gboolean xa_zip_extract (XArchive *archive, GSList *file_list)
 	gchar *password_str, *command;
 	gboolean result;
 
-	files = xa_quote_filenames(file_list, "*?[]", TRUE);
+	files = xa_quote_filenames(file_list, "*?[]", DIR_WITH_ASTERISK);
 	password_str = xa_zip_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[0],
 	                      archive->do_full_path ? "" : " -j",
@@ -253,10 +266,11 @@ void xa_zip_add (XArchive *archive, GSList *file_list)
 		}
 	}
 
-	files = xa_quote_filenames(file_list, NULL, TRUE);   // no escaping for adding!
+	files = xa_quote_filenames(file_list, NULL, DIR_WITH_SLASH);   // no escaping for adding!
 	password_str = xa_zip_password_str(archive);
 	command = g_strconcat(archiver[archive->type].program[1], " -q",
 	                      archive->do_update ? " -ou" : (archive->do_freshen ? " -of" : ""),
+	                      archive->do_recurse ? " -r" : "",
 	                      archive->do_move ? " -m" : "",
 	                      " -", compression,
 	                      epub ? "XD" : "",
@@ -275,7 +289,7 @@ void xa_zip_delete (XArchive *archive, GSList *file_list)
 	GString *files;
 	gchar *command;
 
-	files = xa_quote_filenames(file_list, "*?[]", TRUE);
+	files = xa_quote_filenames(file_list, "*?[]", DIR_WITH_ASTERISK);
 	command = g_strconcat(archiver[archive->type].program[1], " -d ", archive->path[1], " --", files->str, NULL);
 	g_string_free(files,TRUE);
 
